// DCWebUI2.js
// ```````````

var conf = require('./dcwebui-conf.js').conf;

var log = function(sz) {
	var timestamp = function() {
		var d = new Date();
		var pad = function(s) { return s < 10 ? '0'+s : ''+s ; };
		return pad(d.getHours()) + ":" + pad(d.getMinutes());
	};
	console.log("["+timestamp()+"] "+sz.toString());
};

var http_server = function(req, res) {
	var accept = {
		'/'                     : ['/index.htm', 'text/html'],
		'/apple-touch-icon.png' : [false, 'image/png'],
		'/dcwebui.css'          : [false, 'text/css'],
		'/dcwebui.js'           : [false, 'text/javascript'],
		'/sprite-user.png'      : [false, 'image/png'],
		'/favicon.ico'          : [false, 'image/x-icon']
	};
	if (req.url in accept) {
		var path = (accept[req.url][0] === false ? req.url : accept[req.url][0]);
		fs.readFile(
			__dirname + '/client' + path,
			function(err, data) {
				if (err) {
					log('error serving ' + path);
					res.writeHead(500);
					res.end();
				} else {
					log('serving ' + path);
					res.writeHead(200, {
						'Content-Length': data.length,
						'Content-Type'  : accept[req.url][1]
					});
					res.end(data);
				}
			}
		);
	} else if (req.url === '/conf') {
		var ret = 'var DCWEBUI_CONF = ' + JSON.stringify({
			'extern' : conf.web.extern,
			'title'  : conf.web.title
		}) + ";\n";
		res.writeHead(200, {
			'Content-Length': ret.length,
			'Content-Type'  : 'text/javascript'
		});
		res.end(ret);
	} else {
		log('bad request ' + req.url);
		res.writeHead(403);
		res.end('Unauthorised');
	}
};

var sio_server = function(socket) {
	socket.emit('cls');
	socket.emit('raw', ''+conf.app.motd+'<br>');
	socket.emit('sys', "Enter a name to connect as (or name:pass for a registered nick).");
	
	var hub = [];
	
	socket.on('hello', function(data) {
		log('Connection request');
		console.log(data);
		hub = new nmdc.Nmdc({
			'address' : conf.hub.address,
			'port'    : conf.hub.port,
			'tag'     : conf.hub.tag,
			'nick'    : ('nick' in data) ? data.nick.toString() : '',
			'password': ('pass' in data) ? data.pass.toString() : ''
		}, function() {
			socket.emit('hello');
		});
		hub.onSystem = function(s) {
			socket.emit('sys', s);
		};
		hub.onPublic = function(u,m) {
			socket.emit('pub', {'user': u, 'message': m});
		};
		hub.onPrivate = function(u,m) {
			socket.emit('priv', {'user': u, 'message': m});
		};
		hub.onUserPart = function(u) {
			socket.emit('part', {'user': u});
		};
		hub.onUserJoin = function(u) {
			socket.emit('join', {'user': u});
		};
		hub.onClosed = function() {
			socket.emit('sys', 'Connection closed.');
			socket.emit('close');
		};
		hub.onGotHubname = function(s) {
			socket.emit('hubname', s);
		};
		if (conf.app.debug) hub.onDebug = function(s) {
			log(s);
		};
	});	
	socket.on('pub', function(data) {
		hub.say(data.message);
	});
	socket.on('priv', function(data) {
		hub.pm(data.user, data.message);
	});
	socket.on('disconnect', function() {
		log('client dropped');
		if ('disconnect' in hub) hub.disconnect();
	});
};

log(conf.app.name + ' ' + conf.app.version + ' started.');
var http = require('http').createServer(http_server);
var io   = require('socket.io').listen(http);
var fs   = require('fs');
var nmdc = require('nmdc');

io.sockets.on('connection', sio_server);
log('Registered socket.io.');

http.listen(conf.web.port, conf.web.bind_to);
log('Web server started on http://'+conf.web.bind_to+":"+conf.web.port+"/");
